﻿using Windows.Media.Control;
using CS2GSI;
using GlaxLogger;
using Microsoft.Extensions.Logging;
using WindowsMediaController;

namespace CSMediaControl;
using GSI = CS2GSI.CS2GSI;

public class Watchdog : IDisposable, IAsyncDisposable
{
    private bool _keepRunning = true;
    private readonly GSI _gsi;
    private readonly Logger _logger;
    private readonly MediaManager _mediaManager = new();
    private readonly bool _changeState = true;
    private List<MediaManager.MediaSession> _pausedSessions = new();

    public Watchdog(LogLevel? logLevel)
    {
        this._logger = new Logger(logLevel ?? LogLevel.Information, consoleOut: Console.Out);
        this._mediaManager.Start();
        this._gsi = new GSI(this._logger);
        
        this._gsi.OnRoundStart += MusicStop;
        this._gsi.OnRoundOver += MusicStart;
        this._gsi.OnDeath += MusicStart;
        this._gsi.OnHalfTime += MusicStart;
        this._gsi.OnMatchOver += MusicStart;

        this._logger.Log(LogLevel.None, "To pause/resume the program, press P");
        this._logger.Log(LogLevel.None, "To pause all media, press m");
        this._logger.Log(LogLevel.None, "To resume all paused media, press r");
        while (_keepRunning)
        {
            if (Console.KeyAvailable)
            {
                switch (Console.ReadKey(true).Key)
                {
                    case ConsoleKey.P:
                        _changeState = !_changeState;
                        this._logger.Log(LogLevel.Information, $"Execution {(_changeState ? "resumed" : "paused")}");
                        break;
                    case ConsoleKey.M:
                        MusicStop(null);
                        break;
                    case ConsoleKey.R:
                        MusicStart(null);
                        break;
                }
            }else 
                Thread.Sleep(10);
        }
        this._logger.Log(LogLevel.Information, "Bye!");
    }

    private void MusicStop(CS2EventArgs? _)
    {
        if (_changeState)
        {
            this._logger.Log(LogLevel.Information, "Music Pause");
            MediaManager.MediaSession[] playingSessions =
                this._mediaManager.CurrentMediaSessions.Values.Where(session =>
                    session.ControlSession.GetPlaybackInfo().PlaybackStatus ==
                    GlobalSystemMediaTransportControlsSessionPlaybackStatus.Playing).ToArray();
            foreach (MediaManager.MediaSession session in playingSessions)
            {
                this._logger.Log(LogLevel.Debug, $"Pausing {session.Id}");
                session.ControlSession.TryPauseAsync();
            }
            this._pausedSessions = _pausedSessions.Concat(playingSessions).ToList();
        }
    }

    private void MusicStart(CS2EventArgs? _)
    {
        if (_changeState && _pausedSessions.Any())
        {
            this._logger.Log(LogLevel.Information, "Music Start");
            foreach (MediaManager.MediaSession session in _pausedSessions)
            {
                session.ControlSession.TryPlayAsync();
                this._logger.Log(LogLevel.Debug, $"Resuming {session.Id}");
            }
            this._pausedSessions.Clear();
        }
    }   


    public void Dispose()
    {
        this._keepRunning = false;
        _mediaManager.Dispose();
        _logger.Dispose();
    }

    public async ValueTask DisposeAsync()
    {
        this._keepRunning = false;
        _mediaManager.Dispose();
        await _logger.DisposeAsync();
    }
}